import tifffile
from pathlib import Path
import re
import numpy as np
from .normalize import normalize_image

from typing import Tuple

def find_unique_identifier(base_path: Path, identifier=r"\b\d{6}\b", file_ending="*.[tT][iI][fF]") -> Tuple[list, list]:

    all_files = list(base_path.glob(file_ending))
    unique_identifier = list(set([re.findall(identifier, str(x))[0] for x in all_files]))

    return all_files, unique_identifier


def load_rgb(files, identifier=["-r.", "-g.", "-b."]):

    stack = []
    for c in identifier:
        
        file = [x for x in files if c in str(x).lower()][0]
        stack.append(tifffile.imread(file))

    rgb_image = normalize_image(np.stack(stack).transpose(1,2,0))

    return rgb_image


def get_matching_files(all_files, unique_identifier):

    matches = [x for x in all_files if unique_identifier in str(x)]

    return matches


import numpy as np
from scipy.ndimage import binary_dilation
from skimage.morphology import disk

def subtract_background(image, masks, expand_masks=None):
    
    masks[masks != 0] = 1
    
    if expand_masks is not None:
        masks = binary_dilation(masks, iterations=expand_masks, structure=disk(5)).astype(masks.dtype)
        
    background = np.median(image[masks == 0], axis=0)
    
    normed_image = np.clip(image - background, 0, 1)
    normed_image = np.clip((normed_image - normed_image.min(axis=(0,1))) / (np.percentile(image, 99.9, axis=(0,1)) - normed_image.min(axis=(0,1))),  0, 1)

    return normed_image
    
    
def normalize_image(image):
    # Check the data type of the image
    if image.dtype == np.uint8:
        # Normalize uint8 image: scale down to 0-1 range
        return image.astype(np.float32) / 255
    elif image.dtype in (np.float32, np.float64):
        # Check if normalization is needed
        if image.max() > 1:
            return image / 255
        else:
            return image
    else:
        raise TypeError("Unsupported image data type")


from setuptools import setup, find_packages

package_name = "BioUtensils"

def read_requirements():
    with open('requirements.txt', 'r') as file:
        return [line.strip() for line in file.readlines()]

setup(
    name=package_name,
    version='0.4',
    packages=find_packages(),
    install_requires=read_requirements(),
    author="Simon Gutwein",
    include_package_data=True,
    author_email="simon.gutwein@ccri.at",
    description="",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/SimonBon/Bioutensils",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
)